@@ -1,7 +1,5 @@
 //! Crate responsible for managing streamdeck devices, rendering, managing configuration and pretty much everything
 
-/// Thread code used for keeping some tedious work off main threads
-pub mod threads;
 /// Utility code for rendering and conversions
 pub mod util;
 /// Core object and button definitions
@@ -10,9 +8,13 @@ pub mod core;
 pub mod font;
 /// Module definition and built-in modules
 pub mod modules;
+/// Everything related to image processing
+pub mod images;
+
 pub mod versions;
 pub mod config;
 pub mod socket;
+pub mod thread;
 
 pub use streamdeck;
 pub use hidapi;
@@ -23,14 +25,16 @@ use std::sync::Arc;
 use hidapi::HidApi;
 use streamdeck::pids;
 use streamdeck::StreamDeck;
-use crate::config::UniqueDeviceConfig;
+use crate::config::{Config, UniqueDeviceConfig};
 use crate::core::{KeyHandler, SDCore};
 use crate::modules::ModuleManager;
-use crate::threads::rendering::ImageCollection;
+use thread::ImageCollection;
+use crate::socket::SocketManager;
+use crate::thread::rendering::custom::RenderingManager;
 
 /// Attempts to connect to any detected streamdeck
-pub fn connect_any(module_manager: Arc<ModuleManager>, device_config: UniqueDeviceConfig, image_collection: ImageCollection, hid: &HidApi, pool_rate: u32) -> Result<(Arc<SDCore>, KeyHandler), Error> {
-    Ok(SDCore::new(module_manager, device_config, image_collection, attempt_connection_to_any(hid)?, pool_rate))
+pub fn connect_any(module_manager: Arc<ModuleManager>, render_manager: Arc<RenderingManager>, socket_manager: Arc<SocketManager>, config: Arc<Config>, device_config: UniqueDeviceConfig, image_collection: ImageCollection, hid: &HidApi, pool_rate: u32) -> Result<(Arc<SDCore>, KeyHandler), Error> {
+    Ok(SDCore::new(module_manager, render_manager, socket_manager, config, device_config, image_collection, attempt_connection_to_any(hid)?, pool_rate))
 }
 
 fn attempt_connection_to_any(hid: &HidApi) -> Result<StreamDeck, Error> {
@@ -47,8 +51,8 @@ fn attempt_connection_to_any(hid: &HidApi) -> Result<StreamDeck, Error> {
 }
 
 /// Attempts to connect to specified device as a streamdeck
-pub fn connect(module_manager: Arc<ModuleManager>, device_config: UniqueDeviceConfig, image_collection: ImageCollection, hid: &HidApi, vid: u16, pid: u16, serial: &str, pool_rate: u32) -> Result<(Arc<SDCore>, KeyHandler), Error> {
-    Ok(SDCore::new(module_manager, device_config, image_collection, attempt_connection(hid, vid, pid, serial)?, pool_rate))
+pub fn connect(module_manager: Arc<ModuleManager>, render_manager: Arc<RenderingManager>, socket_manager: Arc<SocketManager>, config: Arc<Config>, device_config: UniqueDeviceConfig, image_collection: ImageCollection, hid: &HidApi, vid: u16, pid: u16, serial: &str, pool_rate: u32) -> Result<(Arc<SDCore>, KeyHandler), Error> {
+    Ok(SDCore::new(module_manager, render_manager, socket_manager, config, device_config, image_collection, attempt_connection(hid, vid, pid, serial)?, pool_rate))
 }
 
 fn attempt_connection(hid: &HidApi, vid: u16, pid: u16, serial: &str) -> Result<StreamDeck, Error> {
