@@ -6,10 +6,10 @@ use serde_json::{Error, Value};
 use crate::core::button::Button;
 use crate::core::{ButtonPanel, Panel, RawButtonPanel, UniqueButton, UniqueButtonMap};
 use crate::font::get_font_names;
+use crate::images::SDSerializedImage;
 use crate::modules::components::{UIFieldType, UIFieldValue, UIPathValue, UIValue};
 
-/// Rendering utilities
-pub mod rendering;
+pub use rusttype;
 
 /// Wraps button in [Arc] and [RwLock], but packed into a more convenient function
 pub fn make_button_unique(button: Button) -> UniqueButton {
@@ -68,8 +68,8 @@ pub fn button_to_raw(button: &UniqueButton) -> Button {
     button.read().unwrap().deref().clone()
 }
 
-/// Hashes image blob
-pub fn hash_image(data: &String) -> String {
+/// Hashes string
+pub fn hash_str(data: &String) -> String {
     let mut hasher = DefaultHasher::new();
 
     data.hash(&mut hasher);
@@ -77,6 +77,29 @@ pub fn hash_image(data: &String) -> String {
     hasher.finish().to_string()
 }
 
+/// Hashes image
+pub fn hash_image(data: &SDSerializedImage) -> String {
+    let mut hasher = DefaultHasher::new();
+
+    data.hash(&mut hasher);
+
+    hasher.finish().to_string()
+}
+
+pub fn hash_value<H: Hasher>(value: &Value, state: &mut H) {
+    match value {
+        Value::Null => 0.hash(state),
+        Value::Bool(b) => b.hash(state),
+        Value::Number(n) => n.hash(state),
+        Value::String(s) => s.hash(state),
+        Value::Array(a) => a.iter().for_each(|x| hash_value(x, state)),
+        Value::Object(o) => o.iter().for_each(|(k, v)| {
+            k.hash(state);
+            hash_value(v, state)
+        }),
+    }
+}
+
 /// Converts [UIValue] to [UIPathValue]
 pub fn convert_value_to_path(value: UIValue, current_path: &str) -> UIPathValue {
     let current_path = if current_path.is_empty() { "".to_string() } else { format!("{}.", current_path) };
@@ -93,6 +116,7 @@ pub fn convert_value_to_path(value: UIValue, current_path: &str) -> UIPathValue
                 name: value.name,
                 path,
                 display_name: value.display_name,
+                description: value.description,
                 ty: value.ty,
                 value: UIFieldValue::Collapsable(values)
             }
@@ -112,6 +136,7 @@ pub fn convert_value_to_path(value: UIValue, current_path: &str) -> UIPathValue
                 name: value.name,
                 path,
                 display_name: value.display_name,
+                description: value.description,
                 ty: value.ty,
                 value: UIFieldValue::Array(values)
             }
@@ -124,6 +149,7 @@ pub fn convert_value_to_path(value: UIValue, current_path: &str) -> UIPathValue
                 name: value.name,
                 path,
                 display_name: value.display_name,
+                description: value.description,
                 ty: value.ty,
                 value: value.value.into()
             }
@@ -131,6 +157,13 @@ pub fn convert_value_to_path(value: UIValue, current_path: &str) -> UIPathValue
     }
 }
 
+/// Copies component data from reference to new button
+pub fn straight_copy(reference: &Button, new_button: &mut Button, component_name: &str) {
+    if let Some(data) = reference.0.get(component_name) {
+        new_button.0.insert(component_name.to_string(), data.clone());
+    }
+}
+
 /// Util function to navigate paths and perform changes on values
 pub fn change_from_path<T: Fn(&mut UIValue) -> bool>(path: &str, ui_values: Vec<UIValue>, func: &T, keep: bool) -> (Vec<UIValue>, bool) {
     let mut path = path.split(".");
@@ -208,6 +241,7 @@ pub fn add_array_function() -> fn(&mut UIValue) -> bool {
                 new_item.push(UIValue {
                     name: field.name.clone(),
                     display_name: field.display_name.clone(),
+                    description: field.description.clone(),
                     ty: field.ty.clone(),
                     value: field.default_value.clone()
                 })
